package asd.td2;

import asd.base.PileStatique;

/**
 * Une classe pour l'exercice 5 de la feuille 2.
 *  2001 Marc Gatano
 */
public class Train {

    private PileStatique p; // la voie C
    
    public Train() {
	p = new PileStatique();
    }

    /**
     * Teste si s est une commande valide pour permuter n wagons.
     * Precondition : n > 0.
     * Complexite : O(n).
     */
    public boolean valide(String s, int n) {
	int E = 0; // le nombre de 'E' dans s
	int D = 0; // le nombre de 'D' dans s
	int i = 0;
	
        // INVARIANT :
        // E = le nombre de 'E' dans s[0..i-1]
        // D = le nombre de 'D' dans s[0..i-1]
        // E >= D et E <= n
        // s[0..i-1] n'est constituee que de 'E' et de 'D'
        // VARIANT :
        // i croit et est borne par s.length()
        //
	while ( i < s.length() && E >= D && E <= n ) {
	    switch ( s.charAt(i) ) {
	    case 'E' :
		E++;
		break;
	    case 'D' :
		D++;
		break;
	    default :
		return false;
	    }
	    i++;
	}

	return E == D && E == n;
    }

    public boolean valide(int[] t) {
	boolean[] dejaVu = new boolean[t.length]; // dejaVu[i] == false
                                                  // 0 <= i < t.length
        // INVARIANT :
        // Tout k, 0 <= k < i, apparait une et une seule fois dans t.
        // Pour tout k, 0 <= k < t.length, dejaVu[k] <=> k < i
        // VARIANT :
        // i croit et est majore par t.length.
        //
	for ( int i = 0; i < t.length; i++ ) {
	    if ( 0 <= t[i] && t[i] < t.length && ! dejaVu[t[i]] ) {
		dejaVu[t[i]] = true;
	    }
	    else {
		return false;
	    }
	}

	return true;
    }

    // pour se simplifier la vie

    private boolean vide() {
	return p.vide();
    }

    private void empiler(int i) {
	p.empiler(new Integer(i));
    }

    private void depiler() {
	p.depiler();
    }
    
    private int sommet() {
	return ((Integer) p.sommet()).intValue();
    }

    /**
     * Retourne la commande qui permute le train dont les wagons sont dans
     * l'ordre voieA dans le train dont les wagons sont dans l'ordre voieB.
     * Preconditon : voieA et voieB sont deux permutations de 0,1,..,n-1 ou
     * n = voieA.length = voieB.length et on peut passer de voieA a voieB par
     * une commande.
     * Valeur rendue : la commande qui fait passer de voieA a voieB.
     * Complexite : O(n) ou n = voieA.length = voieB.length.
     */
    public String commande(int[] voieA, int[] voieB) {
	String s = "";
	int i = 0; // pour parcourir voieA
	int j = 0; // pour parcourir voieB

        // INVARIANT :
        // On pose n = voieA.length = voieB.length.
        // 1) Si j < voieB.length alors
        //         voieB[j] = sommet() ou voieB[j] est dans voieA[i..n-1]
        // 2) Si on execute le prefixe s alors
        //         on deplace tous les wagons de voieA[0...i-1] de la voie A
        //         dans p et sur la voie B dans l'ordre voieB[0..j-1]
        // 3) voieB[j..n-1] = p U voieA[i..n-1]
        // VARIANT :
        // j croit et est borne par n.
        //
	while ( j < voieB.length ) {

	    while ( vide() || voieB[j] != sommet() ) {
		empiler(voieA[i]);
		i++;
		s = s + "E";
            }

            depiler();
            j++;
            s = s + "D";
	}

	return s;
    }
}
